Package: vtreat
Type: Package
Title: A Statistically Sound 'data.frame' Processor/Conditioner
Version: 1.6.5
Date: 2024-06-12
Authors@R: c(
    person("John", "Mount", email = "jmount@win-vector.com", role = c("aut", "cre")),
    person("Nina", "Zumel", email = "nzumel@win-vector.com", role = c("aut")),
    person(family = "Win-Vector LLC", role = c("cph"))
    )
URL: https://github.com/WinVector/vtreat/,
        https://winvector.github.io/vtreat/
BugReports: https://github.com/WinVector/vtreat/issues
Maintainer: John Mount <jmount@win-vector.com>
Description: A 'data.frame' processor/conditioner that prepares real-world data for predictive modeling in a statistically sound manner.
    'vtreat' prepares variables so that data has fewer exceptional cases, making
    it easier to safely use models in production. Common problems 'vtreat' defends
    against: 'Inf', 'NA', too many categorical levels, rare categorical levels, and new
    categorical levels (levels seen during application, but not during training). Reference: 
    "'vtreat': a data.frame Processor for Predictive Modeling", Zumel, Mount, 2016, <DOI:10.5281/zenodo.1173313>.
License: GPL-2 | GPL-3
Depends: R (>= 3.4.0), wrapr (>= 2.1.0)
Imports: stats, digest
Suggests: rquery (>= 1.4.99), rqdatatable (>= 1.3.3), data.table (>=
        1.12.2), knitr, rmarkdown, parallel, DBI, RSQLite, datasets,
        R.rsp, tinytest
VignetteBuilder: knitr, R.rsp
RoxygenNote: 7.3.1
ByteCompile: true
NeedsCompilation: no
Packaged: 2024-06-12 15:51:36 UTC; johnmount
Author: John Mount [aut, cre],
  Nina Zumel [aut],
  Win-Vector LLC [cph]
Repository: CRAN
Date/Publication: 2024-06-12 16:40:02 UTC
Built: R 4.6.0; ; 2026-04-26 15:37:39 UTC; unix
