# Generated by roxygen2: do not edit by hand

S3method(apply_right,multinomial_plan)
S3method(apply_right,simple_plan)
S3method(apply_right,treatmentplan)
S3method(apply_right,vtreat_pipe_step)
S3method(apply_transform,vtreat_pipe_step)
S3method(as.character,multinomial_plan)
S3method(as.character,simple_plan)
S3method(as.character,treatmentplan)
S3method(as.character,vtreat_pipe_step)
S3method(as.character,vtreatment)
S3method(as_rquery,vtreat_can_num)
S3method(as_rquery,vtreat_cat_Bayes)
S3method(as_rquery,vtreat_cat_d)
S3method(as_rquery,vtreat_cat_ind)
S3method(as_rquery,vtreat_cat_p)
S3method(as_rquery,vtreat_is_bad)
S3method(as_rquery,vtreat_pass_through)
S3method(as_rquery,vtreatment)
S3method(fit,vtreat_pipe_step)
S3method(fit_prepare,vtreat_pipe_step)
S3method(fit_transform,vtreat_pipe_step)
S3method(format,multinomial_plan)
S3method(format,simple_plan)
S3method(format,treatmentplan)
S3method(format,vtreat_cross_frame_experiment)
S3method(format,vtreat_pipe_step)
S3method(format,vtreatment)
S3method(get_feature_names,vtreat_pipe_step)
S3method(get_score_frame,vtreat_pipe_step)
S3method(get_transform,vtreat_pipe_step)
S3method(prepare,multinomial_plan)
S3method(prepare,simple_plan)
S3method(prepare,treatmentplan)
S3method(prepare,vtreat_pipe_step)
S3method(print,multinomial_plan)
S3method(print,simple_plan)
S3method(print,treatmentplan)
S3method(print,vtreat_cross_frame_experiment)
S3method(print,vtreat_pipe_step)
S3method(print,vtreatment)
export(.wmean)
export(BinomialOutcomeTreatment)
export(MultinomialOutcomeTreatment)
export(NumericOutcomeTreatment)
export(UnsupervisedTreatment)
export(apply_transform)
export(as_rquery_plan)
export(buildEvalSets)
export(center_scale)
export(classification_parameters)
export(designTreatmentsC)
export(designTreatmentsN)
export(designTreatmentsZ)
export(design_missingness_treatment)
export(fit)
export(fit_prepare)
export(fit_transform)
export(flatten_fn_list)
export(getSplitPlanAppLabels)
export(get_feature_names)
export(get_score_frame)
export(get_transform)
export(kWayCrossValidation)
export(kWayStratifiedY)
export(kWayStratifiedYReplace)
export(makekWayCrossValidationGroupedByColumn)
export(materialize_treated)
export(mkCrossFrameCExperiment)
export(mkCrossFrameMExperiment)
export(mkCrossFrameNExperiment)
export(multinomial_parameters)
export(novel_value_summary)
export(oneWayHoldout)
export(patch_columns_into_frame)
export(pre_comp_xval)
export(prepare)
export(problemAppPlan)
export(regression_parameters)
export(rqdatatable_prepare)
export(rquery_prepare)
export(solve_piecewise)
export(solve_piecewisec)
export(spline_variable)
export(spline_variablec)
export(square_window)
export(square_windowc)
export(track_values)
export(unsupervised_parameters)
export(value_variables_C)
export(value_variables_N)
export(variable_values)
export(vnames)
export(vorig)
importFrom(digest,digest)
importFrom(stats,aggregate)
importFrom(stats,anova)
importFrom(stats,as.formula)
importFrom(stats,binomial)
importFrom(stats,chisq.test)
importFrom(stats,fisher.test)
importFrom(stats,glm)
importFrom(stats,lm)
importFrom(stats,lm.wfit)
importFrom(stats,pchisq)
importFrom(stats,pf)
importFrom(stats,quantile)
importFrom(utils,packageVersion)
importFrom(wrapr,"%.>%")
importFrom(wrapr,"%:=%")
importFrom(wrapr,apply_left)
importFrom(wrapr,apply_right)
